/***************************************************************************
 *   Copyright (C) 2004 by Johnathan Burchill                              *
 *   jkerrb@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "diffArchiveThread.h"

#include "archiveHandle.h"
#include "kdar.h"
#include "kdarConfig.h"
#include "kdarSetCursorEvent.h"
#include "kdarStatusBarEvent.h"
#include "kdarWarningEvent.h"

#include <kcursor.h>

#include <dar/libdar.hpp>
#include <string>

extern bool libdarWorking;
extern libdar::archive *theArchive;

diffArchiveThread::diffArchiveThread( kdar * kdarWidget )
{
	widget = kdarWidget;
}

diffArchiveThread::~diffArchiveThread()
{
}

void diffArchiveThread::run()
{
	libdarWorking = true;
	kdarSetCursorEvent *setWaitCursor = new kdarSetCursorEvent( KCursor::waitCursor());
    QApplication::postEvent( widget, setWaitCursor);
	kdarStatusBarEvent *statusEventBegin = new kdarStatusBarEvent( "Comparing archive with filesystem..." );	
	QApplication::postEvent( widget, statusEventBegin );

	//isolate the archive
	libdar::et_mask selection;// = new et_mask();
	selection.add_mask(libdar::bool_mask(true));
	libdar::et_mask subtree;// = new et_mask();
	subtree.add_mask(libdar::bool_mask(true));
	
	libdar::path fs_root( m_kdc->diffArchiveDirectory().latin1() );
	try
	{
		libdar::statistics st=op_diff(theArchive,
				fs_root,
				selection,
				subtree,
				m_kdc->verbose(),
				m_kdc->systemEA(),
				m_kdc->userEA(),
				m_kdc->ignoreID() );
		QString diffInfo;
		QString plural("s do ");
		if (st.errored == libdar::infinint(1)) plural = " does ";
		diffInfo.append(libdar::deci(st.errored).human().c_str()+QString(" file")+plural+QString("not match those on filesystem.\n"));
		plural = "s ";
		if (st.deleted == libdar::infinint(1)) plural = " ";
		diffInfo.append(libdar::deci(st.deleted).human().c_str()+QString(" file")+plural+QString("failed to be read.\n"));
		plural = "s ";
		if (st.ignored == libdar::infinint(1)) plural = " ";
		diffInfo.append(libdar::deci(st.ignored).human().c_str()+QString(" file")+plural+QString("ignored (excluded by filters).\n"));
		plural = "s were ";
		if (libdar::infinint(st.total()) == libdar::infinint(1)) plural = " was ";
		diffInfo.append(QString("A total of ")+libdar::deci(st.total()).human().c_str()+QString(" file")+plural+QString("considered.\n"));
		kdarWarningEvent *warningEvent = new kdarWarningEvent( diffInfo );	
		QApplication::postEvent( widget, warningEvent );
 	}
	catch(libdar::Egeneric &e)
	{
		libdar::user_interaction_warning(e.get_message());
		libdarWorking = false;
	}
	kdarSetCursorEvent *setArrowCursor = new kdarSetCursorEvent( KCursor::arrowCursor());
    QApplication::postEvent( widget, setArrowCursor);
	kdarStatusBarEvent *statusEventEnd = new kdarStatusBarEvent( "Comparing archive with filesystem...done." );	
	QApplication::postEvent( widget, statusEventEnd );
	libdarWorking = false;
}

void diffArchiveThread::setKDarConfig( KDarConfig * kdarconfig )
{
	m_kdc = kdarconfig;
}
