/***************************************************************************
 *   Copyright (C) 2004 by Johnathan Burchill                              *
 *   jkerrb@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "getUserResponseThread.h"
#include "kdar.h"
#include "kdarTarListingEvent.h"
#include "kdarUserInteractionEvent.h"
#include "kdarStatusBarEvent.h"
#include "kdarWarningEvent.h"

#include <kdebug.h>
#include <kmimetype.h>

#include <qpixmap.h>
#include <qregexp.h>
#include <qsemaphore.h>
#include <qthread.h>

extern kdarListViewItem *currentDirectory;
extern bool gotUserResponse;
extern QSemaphore *mySemaphore;

kdar * kdarBase;

getUserResponseThread::getUserResponseThread( kdar * kdarWidget )
	: QThread( 0 )
{
	kdarBase = kdarWidget;
}

getUserResponseThread::~getUserResponseThread()
{
}

void getUserResponseThread::run()
{
}

// The user message callback function required by libdar
void getUserResponseThread::interaction_warning_display(const std::string &message)
{
	//We have to post a custom event here to display the message because we're not the GUI thread.
	QString userMessage = message.c_str();
	//Are we adding a file to the archive? Just post a statusBarEvent
	//because large archive creation slows the GUI responsiveness to a halt!
	//This might be better implemented with a QTimer that checks the number
	//of entries in the message window and deletes 50% of them if
	//the number is larger than some threshold, say 1000 lines...
	//Also, need to implement logging the messages to a logfile.
	
	QRegExp regExp = QRegExp( "^Adding file to archive: " );
	if ( userMessage.contains( regExp ) ) 
	{
		//Post the statusBar event
		userMessage.replace( regExp, "Adding " );
		kdarStatusBarEvent *statusEvent = new kdarStatusBarEvent( userMessage );
		QApplication::postEvent( kdarBase, statusEvent );
	}
	else
	{
		kdarWarningEvent *warnEvent = new kdarWarningEvent( userMessage );	
		QApplication::postEvent( kdarBase, warnEvent );
	}
}

//This is the tar listing callback used by libdar
void getUserResponseThread::tarListing( const std::string & flag,
										const std::string & perm,
										const std::string & uid,
										const std::string & gid,
										const std::string & size,
										const std::string & date,
										const std::string & filename)
{
	//We have to post a custom event here to display the listing because we're not the GUI thread.
	QString qflag = flag.c_str();
	QString qperm = perm.c_str();
	QString quid = uid.c_str();
	QString qgid = gid.c_str();
	QString qsize = size.c_str();
	QString qdate = date.c_str();
	QString qfilename = filename.c_str();
	QString myExpandDirectory = kdarBase->expandDirectory();
	//kdDebug() << "URL: " << QString(myExpandDirectory + '/' + qfilename).latin1() << endl;
	QString url = '/' + myExpandDirectory + '/' + qfilename;
	if (qperm[0] == 'd')
	{
		url = '/';
	}
	QPixmap qicon = KMimeType::pixmapForURL(url, 0, KIcon::Small); 
	//create and post a new tarListingEvent
	kdarTarListingEvent *tarListingEvent = new kdarTarListingEvent( currentDirectory, qflag, qperm, quid, qgid, qsize, qdate, qfilename, qicon);	
	QApplication::postEvent( kdarBase, tarListingEvent );
}

//This is the user interaction callback function required by libdar
bool getUserResponseThread::interaction_pause(const std::string &message)
{
	//Don't access the GUI directly here because we're not the GUI thread.
	QString userQuestion = message.c_str();
	kdarUserInteractionEvent *questionEvent = new kdarUserInteractionEvent( userQuestion );
	//reset the gotUserResponse boolean:
	gotUserResponse = false;
	//post the questionEvent
	QApplication::postEvent( kdarBase, questionEvent );
	kdDebug() << "interaction_pause: going to sleep..."  << endl;
	(*mySemaphore)++;
	kdDebug() << "interaction_pause: waking up!" << endl;
	kdDebug() << "interaction_pause: User answered " << gotUserResponse << "." << endl;
	return gotUserResponse;
}

void getUserResponseThread::setKDarWidget( kdar * kdarBaseWidget )
{
	kdarBase = kdarBaseWidget;
}
