/*
 * Copyright (C) 2003,2004 Johnathan Burchill <jkerrb@users.sourceforge.net>
 */

#ifndef _KDAR_H_
#define _KDAR_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif


#include <qstring.h>
#include <qthread.h>
#include <qmutex.h>
#include <qwaitcondition.h>

#include "kdarcreatedialog.h"
#include "kdarextractdialog.h"
#include "kdarcreatearchivewizard.h"

#include <kaction.h>
#include <kapplication.h>
#include <kmainwindow.h>

#include <string>

#include <dar/libdar.hpp>

const char* const EXTENSION = "dar";
const QString FILENAME("Filename");
const QString PERMISSIONS("Permissions");
const QString USERID("User ID");
const QString GROUPID("Group ID");
const QString SIZE("Size");
const QString DATE("Date");
const QString STATUS("Status");

const int DURATION( 5000 ); // milliseconds for messages in statusbar
const int CHECKINTERVAL( 2000 );
class KPrinter;
class KToggleAction;
class KURL;
class KURLPixmapProvider;
class KConfig;
class KDarConfig;
class kdarView;
class QTimer;

class createArchiveThread;
class diffArchiveThread;
class isolateArchiveThread;
class listArchiveThread;
class openArchiveThread;
class populateTreeArchiveThread;
class restoreArchiveThread;
class testArchiveThread;
class getUserResponseThread;

/**
 * This class serves as the main window for kdar.  It handles the
 * menus, toolbars, and status bars.
 *
 * @short Main window class
 * @author $AUTHOR <$EMAIL>
 * @version $APP_VERSION
 */
class kdar : public KMainWindow
{
    Q_OBJECT
public:
    
	kdar();
    virtual ~kdar();

    void openArchive( KDarConfig * kdconfig = 0);
	void closeArchive();
    void setExpandDirectory(const QString & expandDirectory) { m_expandDirectory = expandDirectory; }
	QString expandDirectory() const { return m_expandDirectory; }
	
	virtual void dropEvent(QDropEvent *event);

    kdarCreateDialog *createDlg;
    kdarCreateArchiveWizard *createDlgWizard;
	KActionMenu *contextPopupMenu;
    QString newArchiveDirectory;
	KURLPixmapProvider *kdarPixmapProvider;
	createArchiveThread *createThread;
	diffArchiveThread *diffThread;
	isolateArchiveThread *isolateThread;
	listArchiveThread *listThread;
	openArchiveThread *openThread;
	populateTreeArchiveThread *populateTreeThread;
	restoreArchiveThread *restoreThread;
	testArchiveThread *testThread;
	getUserResponseThread *userResponseThread;

protected:
    /**
     * Overridden virtuals for Qt drag 'n drop (XDND)
     */
    virtual void dragEnterEvent(QDragEnterEvent *event);

protected:
    /**
     * This function is called when it is time for the app to save its
     * properties for session management purposes.
     */
    void saveProperties();

    /**
     * This function is called when this app is restored.  The KConfig
     * object points to the session management config file that was saved
     * with @ref saveProperties
     */
    void readProperties();

	virtual bool event( QEvent *e );
	
public slots:
    void slotArchiveList( const bool loadArchive = true );
	void slotArchiveChoose( const QString& text = QString::null );
	void slotArchiveChoose( const KURL & );
	
private slots:
    void myquit();
    void filePrint();
    void optionsShowToolbar();
    void optionsShowStatusbar();
    void optionsConfigureKeys();
    void optionsConfigureToolbars();
    void optionsPreferences();
    void newToolbarConfig();

    void changeStatusbar( const QString& text );
    void changeCaption( const QString& text );

    //some slots
    void slotArchiveCreate();
    void slotArchiveExtract();
    void slotArchiveIsolate();
    void slotArchiveTest();
    void slotArchiveDiff();
	void slotClearMessages();
	void slotTerminateOperation();

	void slotConfigureCreationSettings();
	void slotConfigureRestoreSettings();
	void slotSaveCreationProfile();
	void slotRestoreCreationProfile();
	void slotSaveRestoreProfile();
	void slotRestoreRestoreProfile();

	void slotCreationCheckFileSize();

private:
    void setupAccel();
    void setupActions();
	void updateGUI();
	long lastArchiveSliceNumber( const QString & archiveBaseName );

private:
    kdarView *m_view;

    KPrinter   *m_printer;
    KToggleAction *m_toolbarAction;
    KToggleAction *m_statusbarAction;
	KRecentFilesAction *m_recentFilesAction;
	
    KDarConfig *m_kdarconfig;
    KDarConfig *m_createConfig;
	KDarConfig *m_restoreConfig;
	kdarExtractDialog *extractDlg;
	QString m_expandDirectory;
	QTimer *fileSizeTimer;
};

#endif // _KDAR_H_
