/***************************************************************************
 *   Copyright (C) 2004 by Johnathan Burchill                              *
 *   jkerrb@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifndef _KDARCONFIG_H_
#define _KDARCONFIG_H_

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "kdarconfigobject.h"
#include "kdarcompressionfilterdialog.h"
#include "kdardirectoryfilterdialog.h"
#include "kdarfilefilterdialog.h"

#include <qfont.h>
#include <qwidget.h>
#include <qstring.h>
#include <qstringlist.h>
#include <string>

#include "kdarEnums.h"
#include <dar/libdar.hpp>


class KConfig;
class QListView;

/**
 * A configuration class/dialog for KDar.
 *
 */

class KDarConfig : public KDarConfigObject
{
  Q_OBJECT

  public:
    
	KDarConfig( QWidget *parent=0, const char *name=0,
		  KDarConfig *kdarConfig=0, bool addHelpButton = false );

    KDarConfig( const KDarConfig & );

    virtual ~KDarConfig();

    void operator=( const KDarConfig &kdc );

    /**
	 * Options storing routines.
	 */
	 	
	void setGlobalConfig( KConfig * kconfig );

	void setArchiveName( const QString & archiveName );
	void setArchiveStorageDirectory( const QString & archiveStorageDirectory );
	void setNewArchiveName( const QString & newArchiveName );
	void setDirectoryToBackup( const QString & directoryToBackup );
	void setDifferentialBackup( const bool differentialBackup );
	void setDifferentialBackupArchiveName( const QString & differentialBackupArchiveName );
	void setInputPipe( const QString & inputPipe );	
	void setOutputPipe( const QString & outputPipe );	
	void setExecuteCommand( const QString & executeCommand );
	void setPasswordString( const QString & passwordString );
	void setExecuteRef( const QString & executeRef );
	void setPassRef( const QString &passRef );
	
	void setVerbose( const bool verbose );
	void setAllowOverwrite( const bool allowOverwrite );
	void setWarnOnOverwrite( const bool warnOnOverwrite );
	void setUserEA( const bool userEA );
	void setSystemEA( const bool systemEA );
	void setIgnoreID( const bool ignoreID );
	void setNoDelete( const bool noDelete );
	void setRestoreRecent( const bool restoreRecent );
	void setFlatRestore( const bool flatRestore );
	
	void setUseCreateArchiveWizard( const bool useCreateArchiveWizard );
	void setSplitArchive( const bool splitArchive );
	void setPauseBetweenSlices( const bool pauseBetweenSlices );
	void setKeepPruned( const bool keepPruned );
	void setIgnoreDump( const bool ignoreDump );
	
	void setMedia( const int media );
	void setSliceSizeString( const QString & sliceSizeString );
	void setSliceSizeSuffix( const int sliceSizeSuffix );
	void setFirstSliceSizeString( const QString & firstSliceSizeString );
	void setFirstSliceSizeSuffix( const int firstSliceSizeSuffix );
	void setCompressionAlgorithm( const int compressionAlgorithm );
	void setCompressionLevel( const int compressionLevel );
	void setMinimumCompressionSizeInt( const int minimumCompressionSizeInt );
	void setHourShiftInt( const int hourShiftInt );
	
	void setCompressionMaskList( const QStringList & compressionMaskList );	
	void setDirectoryMaskList( const QStringList & directoryMaskList );	
	void setFileMaskList( const QStringList & fileMaskList );	
	void setCompressionMaskType( const int compressionMaskType );
	void setDirectoryMaskTypes( const QStringList & directoryMaskTypes );
	void setFileMaskType( const int fileMaskType );
	
	void setGeneralFont( const QFont & generalFont );
	void setFileBrowserFont( const QFont & fileBrowserFont );
	void setMessageWindowFont( const QFont & messageWindowFont );
	void setStatusBarFont( const QFont & statusBarFont );

	//members with no corresponding GUI --- used for storage 
	// for create and restore dialogs. I.e. these aren't used globally.
	void setDryRun( const bool dryRun );
	void setIsolationArchiveName( const QString & isolationArchiveName );
	void setDiffArchiveDirectory( const QString & diffArchiveDirectory );
	void setExtractArchiveDirectory( const QString & extractArchiveDirectory );
	void setNewArchiveBasename( const QString & newArchiveBasename );
	/**
     * Options reading routines.
     */
    
	KConfig * globalConfig() const;
	QString archiveName() const;
	QString archiveStorageDirectory() const;
	QString newArchiveName( ) const;
	QString directoryToBackup( ) const;
	bool differentialBackup( ) const;
	QString differentialBackupArchiveName( ) const;
	QString inputPipe() const;
	QString outputPipe() const;
	QString executeCommand() const;
	QString passwordString() const;
	QString executeRef() const;
	QString passRef() const;
	
	bool verbose() const;
	bool allowOverwrite() const;
	bool warnOnOverwrite() const;
	bool userEA() const;
	bool systemEA() const;
	bool ignoreID() const;
	bool noDelete() const;
	bool restoreRecent() const;
	bool flatRestore() const;
	
	bool useCreateArchiveWizard() const;
	bool splitArchive() const;
	bool pauseBetweenSlices() const;
	bool keepPruned() const;
	bool ignoreDump() const;
	int media() const;
	QString sliceSizeString() const;
	int sliceSizeSuffix() const;
	QString firstSliceSizeString() const;
	int firstSliceSizeSuffix() const;
	int compressionAlgorithm() const;
	int compressionLevel() const;
	int minimumCompressionSizeInt() const;
	int hourShiftInt() const;

	QStringList compressionMaskList() const;	
	QStringList directoryMaskList() const;	
	QStringList fileMaskList() const;
	int compressionMaskType() const;	
	QStringList directoryMaskTypes() const;
	int fileMaskType() const;
	
	QFont generalFont() const;
	QFont fileBrowserFont() const;
	QFont messageWindowFont() const;
	QFont statusBarFont() const;
    
	//Non-gui members
	bool dryRun() const;
	QString isolationArchiveName() const;
	QString diffArchiveDirectory() const;
	QString extractArchiveDirectory() const;
	QString newArchiveBasename() const;
	
	libdar::infinint libdarSliceSize();
	libdar::infinint libdarFirstSliceSize();
	libdar::et_mask libdarCreateCompressionMask();
	libdar::et_mask libdarCreateFileMask();
	libdar::et_mask libdarCreateDirectoryMask();
		
	/**
     * Call this method before this class is deleted  if you want
     * the settings you have (or the user has) chosen to become the
     * global, default settings.
     */
    bool writeGlobalSettings();

    bool readGlobalSettings();
	void updateGUI();  

protected:
    void fillInDialog();
	void fillListView( QListView *, QStringList );
	void loadDirectoryFilters();
	QStringList makeQStringList( QListView * listView, int column = 0 );
	void makeDirectoryQStringLists( QListView * listView, QStringList & directories, QStringList & types );


public slots:
    /**
     * Use this function to activate the help information for this
     * widget. The function is particulary useful if the help button is
     * not displayed as specified by the constructor. Normally you want
     * to hide the help button if this widget is embedded into a larger
     * dialog box that has its own help button. See kedit
     * (optiondialog.cpp) for an example
     */
    void activateHelp( void );
	void setMediaSize( int );


//protected slots:
    /**
     * Invokes the help documentation for kspell
     */ 
	

	virtual void slotArchiveName( const QString & );
	virtual void slotArchiveStorageDirectory( const QString & );
	virtual void slotInputPipe( const QString & );	
	virtual void slotOutputPipe( const QString & );	
	virtual void slotExecuteCommand( const QString & );
	virtual void slotPasswordString( const QString & );
	virtual void slotExecuteRef( const QString & );
	virtual void slotPassRef( const QString & );
	
	virtual void slotVerbose( bool );
	virtual void slotAllowOverwrite( bool );
	virtual void slotWarnOnOverwrite( bool );
	virtual void slotUserEA( bool );
	virtual void slotSystemEA( bool );
	virtual void slotIgnoreID( bool );
	virtual void slotNoDelete( bool );
	virtual void slotRestoreRecent( bool );
	virtual void slotFlatRestore( bool );
	
	virtual void slotUseCreateArchiveWizard( bool );
	virtual void slotSplitArchive( bool );
	virtual void slotPauseBetweenSlices( bool );
	virtual void slotKeepPruned( bool );
	virtual void slotIgnoreDump( bool );
	virtual void slotMedia( int );//#include "kdarconfigureobject.h"

	virtual void slotSliceSizeString( const QString & );
	virtual void slotSliceSizeSuffix( int );
	virtual void slotFirstSliceSizeString( const QString & );
	virtual void slotFirstSliceSizeSuffix( int );
	virtual void slotCompressionAlgorithmBZip2( bool );
	virtual void slotCompressionAlgorithmGZip( bool );
	virtual void slotCompressionAlgorithmNone( bool );
	virtual void slotCompressionLevel( int );
	virtual void slotMinimumCompressionSizeInt( int );
	virtual void slotHourShiftInt( int );

	virtual void slotRemoveAllFileFilter();
	virtual void slotRemoveFileFilter();
	virtual void slotEditFileFilter();
	virtual void slotNewFileFilter();
	virtual void slotRemoveAllDirectoryFilter();
	virtual void slotRemoveDirectoryFilter();
	virtual void slotEditDirectoryFilter();
	virtual void slotNewDirectoryFilter();
	virtual void slotRemoveAllCompressionFilter();
	virtual void slotRemoveCompressionFilter();
	virtual void slotEditCompressionFilter();
	virtual void slotNewCompressionFilter();
	virtual void slotUpdateCompressionMaskList();
	virtual void slotUpdateDirectoryMaskList();
	virtual void slotUpdateFileMaskList();
	
	virtual void slotCompressionMaskList( const QStringList & );	
	virtual void slotDirectoryMaskList( const QStringList & );	
	virtual void slotFileMaskList( const QStringList & );	
	virtual void slotCompressionMaskType( int );
	virtual void slotDirectoryMaskTypes( const QStringList & );
	virtual void slotFileMaskType( int );
		
	virtual void slotGeneralFont( const QFont & );
	virtual void slotFileBrowserFont( const QFont &);
	virtual void slotMessageWindowFont( const QFont &);
	virtual void slotStatusBarFont( const QFont &);


protected slots:

	void slotHelp();

protected:

    // The options
    bool m_nodialog;
	KConfig *m_kc;
	kdarCompressionFilterDialog *m_compressionDlg;
	kdarDirectoryFilterDialog *m_directoryDlg;
	kdarFileFilterDialog *m_fileDlg;
		
	QStringList mediaNames;
	QStringList mediaSizes;
	QStringList mediaFirstSizes;
	int mediaSuffices[17];
	int mediaFirstSuffices[17];


	QString m_archiveName;
	QString m_archiveStorageDirectory;
	QString m_newArchiveName;
	QString m_directoryToBackup;
	bool m_differentialBackup;
	QString m_differentialBackupArchiveName;
	QString m_inputPipe;
	QString m_outputPipe;
	QString m_executeCommand;
	QString m_passwordString;
	QString m_executeRef;
	QString m_passRef;
	
	bool m_verbose;
	bool m_allowOverwrite;
	bool m_warnOnOverwrite;
	bool m_userEA;
	bool m_systemEA;
	bool m_ignoreID;
	bool m_noDelete;
	bool m_restoreRecent;
	bool m_flatRestore;
	
	bool m_useCreateArchiveWizard;
	bool m_splitArchive;
	bool m_pauseBetweenSlices;
	bool m_keepPruned;
	bool m_ignoreDump;
	int m_media;
	QString m_sliceSizeString;
	int m_sliceSizeSuffix;
	QString m_firstSliceSizeString;
	int m_firstSliceSizeSuffix;
	int m_compressionAlgorithm;
	int m_compressionLevel;
	int m_minimumCompressionSizeInt;
	int m_hourShiftInt;

	QStringList m_compressionMaskList;	
	QStringList m_directoryMaskList;	
	QStringList m_fileMaskList;
	int m_compressionMaskType;
	QStringList m_directoryMaskTypes;
	int m_fileMaskType;
	
	QFont m_generalFont;
	QFont m_fileBrowserFont;
	QFont m_messageWindowFont;
	QFont m_statusBarFont;


signals:
    void configChanged();
	void compressionMaskListChanged();
	void directoryMaskListChanged();
	void fileMaskListChanged();

private:
	//non-gui members stored in KDarConfig class
	bool m_dryRun;
	QString m_isolationArchiveName;
	QString m_diffArchiveDirectory;
	QString m_extractArchiveDirectory;
	QString m_newArchiveBasename;
};

#endif // _KDARCONFIG_H_
