/***************************************************************************
 *   Copyright (C) 2003,2004 by Johnathan Burchill                         *
 *   jkerrb@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/


#include "kdarcreatedialog.h"
#include "kdar.h"
#include "kdarConfig.h"

#include <kdebug.h>
#include <kfiledialog.h>
#include <klineedit.h>
#include <kurl.h>
#include <kurlrequester.h>

#include <qcheckbox.h>
#include <qcombobox.h>
#include <qlabel.h>
#include <qlineedit.h>
#include <qlistview.h>
#include <qpushbutton.h>
#include <qradiobutton.h>
#include <qstring.h>
#include <qtabwidget.h>
#include <qvbox.h>
#include <qlayout.h>

#include <iostream>

QString newArchiveName, referenceArchive;

kdarCreateDialog::kdarCreateDialog( kdar* kdarWidget, QWidget* parent, const char* name, bool modal, WFlags fl)
: archiveCreateDlg(parent,name, modal,fl),
	directoryDlg( new kdarDirectoryFilterDialog),
	fileDlg( new kdarFileFilterDialog ),
	compressionDlg( new kdarCompressionFilterDialog )
	
{
	m_kdar = kdarWidget;
	rootDirectoryKURLRequester->setMode(26); //existing local directory
	rootDirectoryKURLRequester->setCaption("Select the directory you want to backup:");
	archiveNameKURLRequester->setMode(17); //local file
	archiveNameKURLRequester->setCaption("Select a name for this archive:");
	referenceArchiveKURLRequester->setMode(25); //existing local file
	referenceArchiveKURLRequester->setFilter("*.dar"); //dar archives
	referenceArchiveKURLRequester->setCaption("Select an archive to use as a reference for this backup:");
}

kdarCreateDialog::~kdarCreateDialog()
{
}

/*$SPECIALIZATION$*/
void kdarCreateDialog::setKDarConfig( KDarConfig* createConfig )
{
	m_createConfig = createConfig;
}

void kdarCreateDialog::slotChooseRootDirectory(const QString & theSelection)
{
	if (theSelection != NULL)
	{
		archiveNameLabel->setEnabled(true);
		archiveNameKURLRequester->setEnabled(true);
		slotChooseArchiveName( archiveNameKURLRequester->lineEdit()->text() );
	}
}

void kdarCreateDialog::slotChooseArchiveName( const QString & theSelection )
{
	if ( !theSelection.isEmpty() )
	{
		diffBackupCheckBox->setEnabled( true );
	}
	else
	{
		diffBackupCheckBox->setEnabled( false );
	}
	//automatically exclude the storage directory from the backup if the
	//directory is a child of the directory to back up.
	
	//The directory mask type "auto" will be used to signify that this mask
	//is automatically generated, and the user cannot edit or remove it.
	
	//First, if theSelection is empty, we have no archivename yet, so 
	//the auto filter should not be in the list of directory filters. 
	//If theSelection is not empty, we need to replace the current Auto filter.
	//In either case, remove the auto filter if it exists by traversing the list.
	QListViewItem * currentItem = m_createConfig->directoryFiltersListView->firstChild();
	while( currentItem )
	{
		if ( currentItem->text( 0 ) == "Auto" )
		{
			kdDebug() << "kdarCreateDialog::slotChooseArchiveName() : deleting old auto filter." << endl;
			delete currentItem;
			currentItem = 0;
		}
		if ( currentItem )
		{
			currentItem = currentItem->nextSibling();
    	}
	}
	
	if ( !currentItem )
	{
		//We deleted the "Auto" filter, so update the lists
		m_createConfig->slotUpdateDirectoryMaskList();
	}
	
	if ( !theSelection.isEmpty() )
	{
		QString rootDirectory = rootDirectoryKURLRequester->lineEdit()->text();
		QString newArchiveName( theSelection );
		//If theSelection doesn't start with '/', the user wants to override
		//the default storage directory.
		if ( theSelection[0] != '/' )
		{
			KURL archiveStorageDirKURL( m_createConfig->archiveStorageDirectory() );
			newArchiveName = archiveStorageDirKURL.path( 1 ) + theSelection;
		}
		//We only add the storage directory to filter list if it is a child of rootDirectory
		if ( newArchiveName.left( rootDirectory.length() ) == rootDirectory )
		{
			//the new archive name is in a subdirectory of
			//the root directory, so put the new archive directory
			//in the list of excluded paths
			KURL newArchiveKURL = newArchiveName;
			QString newArchiveDirectory = newArchiveKURL.directory(true, false);
			
			if ( !newArchiveDirectory.isEmpty() )
			{
				//Add the filter
				QString type( "Auto" );
				kdDebug() << "kdarCreateDialog::slotChooseArchiveName() 12: adding new Auto directory filter: " << newArchiveDirectory.latin1() << endl;
				QListViewItem* item = new QListViewItem( m_createConfig->directoryFiltersListView, type, newArchiveDirectory );
				//Disable this item. It is automatically added/removed based
				//on root directory and newArchiveName, so don't let the user 
				//change it.
				item->setEnabled( false );
				m_createConfig->slotUpdateDirectoryMaskList();
			}
		}
	}
}

void kdarCreateDialog::slotRemoveCompressionFilter()
{
	//QListViewItem * currItem = compressionFiltersListView->selectedItem();
	//if (!currItem) return;
	//delete currItem;
}

void kdarCreateDialog::slotRemoveAllCompressionFilter()
{
	//compressionFiltersListView->clear();
}

void kdarCreateDialog::slotEditCompressionFilter()
{
}

void kdarCreateDialog::slotNewCompressionFilter()
{
	if (compressionDlg->exec())
	{
		//QString compressionMask = compressionDlg->compressionFilterValueComboBox->currentText();
/*		if (compressionMask != QString(""))
		{
			if (compressionMask != QString("all of the above"))
			{
				QString type = "FIX ME!";
				//compressionDlg->compressionFilterTypeComboBox->currentText();
				(void) new QListViewItem(compressionFiltersListView, type, compressionMask);
			}
			else
			{
				QString type = "FIX ME!"; 
				//compressionDlg->compressionFilterTypeComboBox->currentText();
				for (int i = 0; i<(compressionDlg->compressionFilterValueComboBox->count()-1);++i)
				{
					compressionMask = compressionDlg->compressionFilterValueComboBox->text(i);
					(void) new QListViewItem(compressionFiltersListView, type, compressionMask);
				}
			}
		}
		else
		{
			std::cout << "Mask expression not specified." << rootDirectory.latin1() << std::endl;
		}
*/	
	}
}

void kdarCreateDialog::slotRemoveDirectoryFilter()
{
/*	QListViewItem * currItem = directoryFiltersListView->selectedItem();
	if (!currItem) return;
	delete currItem;
*/
}

void kdarCreateDialog::slotRemoveAllDirectoryFilter()
{
	//directoryFiltersListView->clear();
}


void kdarCreateDialog::slotRemoveFileFilter()
{
/*	QListViewItem * currItem = fileFiltersListView->selectedItem();
	if (!currItem) return;
	delete currItem;
*/
}

void kdarCreateDialog::slotRemoveAllFileFilter()
{
//	fileFiltersListView->clear();
}

void kdarCreateDialog::slotEditFileFilter()
{
/*	QListViewItem * currItem = fileFiltersListView->currentItem();
	if (!currItem) return;

	QString text = currItem->text(0);
	if (text == QString("Include"))
	{
		fileDlg->fileFilterTypeComboBox->setCurrentItem(0);
	}
	else if (text == QString("Exclude"))
	{
			fileDlg->fileFilterTypeComboBox->setCurrentItem(1);
	}
	fileDlg->fileFilterValueLineEdit->setText(currItem->text(1));

	if (fileDlg->exec())
	{
		QString subdir = fileDlg->fileFilterValueLineEdit->text();
		if (subdir != rootDirectory)
		{
			QString type = fileDlg->fileFilterTypeComboBox->currentText();
			currItem->setText(0, type);
			currItem->setText(1, subdir);
		}
		else
		{
			std::cout << "the subdir does not belong to " << rootDirectory.latin1() << std::endl;
		}
	}
*/
}

void kdarCreateDialog::slotEditDirectoryFilter()
{
/*	QListViewItem * currItem = directoryFiltersListView->currentItem();
	if (!currItem || !currItem->isSelected()) return;
	
	int type;
	QString text = currItem->text(0);
	if (text == QString("Include"))
	{
		type = 1;
	}
	else
	{
		type = 0;
	}
	if (directoryDlg->listRadioButton->isChecked())
	{
		directoryDlg->directoryListTypeComboBox->setCurrentItem(type);
	}
	else
	{
		directoryDlg->directoryFilesystemTypeComboBox->setCurrentItem(type);
	}
	directoryDlg->directoryFilterValueLineEdit->setText(currItem->text(1));

	if (directoryDlg->exec())
	{
		if (directoryDlg->listRadioButton->isChecked())
		{
			QString subDir = directoryDlg->directoryListComboBox->currentText();
			if (subDir != QString("") && subDir != QString("All of the above") && subDir != rootDirectory)
			{
				QString filterType = directoryDlg->directoryListTypeComboBox->currentText();
				currItem->setText(0, filterType);
				currItem->setText(1, subDir);
			}		
		}
		else
		{
			QString subdir = directoryDlg->directoryFilterValueLineEdit->text();
			if (subdir != rootDirectory)
			{
				QString filterType = directoryDlg->directoryFilesystemTypeComboBox->currentText();
				currItem->setText(0, filterType);
				currItem->setText(1, subdir);
			}
		}
	}
*/
}

void kdarCreateDialog::slotNewFileFilter()
{
/*	if (fileDlg->exec())
	{
		QString filter = fileDlg->fileFilterValueLineEdit->text();
		if (filter != "")
		{
			QString type = fileDlg->fileFilterTypeComboBox->currentText();
			(void) new QListViewItem(fileFiltersListView, type, filter);
		}
		else
		{
			#ifdef DEBUG
				std::cout << "empty file filter" << std::endl;
			#endif
		}
	}
*/
}

void kdarCreateDialog::slotNewDirectoryFilter()
{
/*	if (directoryDlg->exec())
	{
		if (directoryDlg->listRadioButton->isChecked())
		{
			QString subDir;
			subDir = directoryDlg->directoryListComboBox->currentText();
			if (subDir != QString(""))
			{
				if (subDir != QString("All of the above"))
				{
					QString type = directoryDlg->directoryListTypeComboBox->currentText();
					(void) new QListViewItem(directoryFiltersListView, type, rootDirectory+subDir);
				}
				else
				{
					QString type = directoryDlg->directoryListTypeComboBox->currentText();
					for (int i = 0; i<(directoryDlg->directoryListComboBox->count()-1);++i)
					{
						subDir = directoryDlg->directoryListComboBox->text(i);
						(void) new QListViewItem(directoryFiltersListView, type, rootDirectory+subDir);
					}
				}
			}		
		}
		else
		{	
			QString subdir = directoryDlg->directoryFilterValueLineEdit->text();
			if (subdir != rootDirectory)
			{
				QString type = directoryDlg->directoryFilesystemTypeComboBox->currentText();
				(void) new QListViewItem(directoryFiltersListView, type, subdir);
			}
			else
			{
				std::cout << "the subdir does not belong to " << rootDirectory.latin1() << std::endl;
			}
		}
	}
*/
}




#include "kdarcreatedialog.moc"

