/***************************************************************************
 *   Copyright (C) 2003, 2004 by Johnathan Burchill                        *
 *   jkerrb@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/


#include "kdardialog.h"

#include "kdarConfig.h"
#include "kdarListViewItem.h"
#include "kdarWarningEvent.h"
#include "kdarUserInteractionEvent.h"
#include "kdar.h"
#include "archiveHandle.h"
#include "listArchiveThread.h"
#include "openArchiveThread.h"
#include "populateTreeArchiveThread.h"

#include <qcombobox.h>
#include <qevent.h>
#include <qdragobject.h>
#include <kaction.h>
#include <kcursor.h>
#include <kdebug.h>
#include <kfiledialog.h>
#include <klistview.h>
#include <kmessagebox.h>
#include <ktextbrowser.h>
#include <kurl.h>

#include <iostream>
#include <string>
#include <dar/libdar.hpp>
#include <dar/tools.hpp>

extern libdar::archive *theArchive;
extern kdarListViewItem *currentDirectory;
extern kdarListViewItem * myExpandChild;
extern bool libdarWorking;

kdarDialog::kdarDialog(QWidget* parent, const char* name, WFlags fl)
: KDARDlg(parent,name,fl)
{
	setAcceptDrops(true);
	//archiveFileView->setEnabled(false);

	archiveFileView->setAcceptDrops(true);
	//Set right alignment on all columns except the first
	int numCols = archiveFileView->columns();
	for( int i = 1; i < numCols; ++i)
	{
		archiveFileView->setColumnAlignment( i, Qt::AlignRight);	
	}
	//set the kdar pointer
	m_kdar = static_cast<kdar*>(parentWidget()->parentWidget());
}

kdarDialog::~kdarDialog()
{
}

/*$SPECIALIZATION$*/
void kdarDialog::slotArchiveChoose(const QString & text)
{
	if (text[0] == '/')
	{
		m_kdc->setArchiveName( text );
//		archiveHandle archiveHndl(text);
//		*ref_root = archiveHndl.getArchiveLibdarPath();
//		ref_filename = archiveHndl.getArchiveBasenameString();
		if (theArchive != 0) m_kdar->closeArchive();
		try
		{
			m_kdar->openArchive();
		}
		catch (libdar::Egeneric &e)
		{
			libdar::user_interaction_warning(e.get_message());
			if (theArchive != 0) delete theArchive;
			theArchive = 0;
		}
	}
}

void kdarDialog::slotArchiveChoose()
{
}

void kdarDialog::slotArchiveDescend(QListViewItem * expandItem)
{
	if (libdarWorking)
	{
		return;
	}
	if (m_kdar->openThread->running()) m_kdar->openThread->wait();
	if (theArchive == 0) return;

	kdDebug() << "Expanding " << expandItem->text(0).latin1() << endl;
	myExpandChild = static_cast<kdarListViewItem*>(expandItem->firstChild());
	m_kdar->populateTreeThread->start();
}

void kdarDialog::slotArchiveContextMenu(KListView *, QListViewItem* contextItem, const QPoint & point )
{
	if (contextItem != 0)
	{
		kdarListViewItem * kdarContextItem = static_cast<kdarListViewItem*>(contextItem);
		kdDebug() << "Context menu item: " << kdarContextItem->text(0).latin1() << endl;
		m_kdar->contextPopupMenu->popup(point);
	}
}

void kdarDialog::dragEnterEvent(QDragEnterEvent *event)
{
    // accept uri drops only
    kdDebug() << "kdarDialog::dragEnterEvent reached here!" << endl;
    if (QUriDrag::canDecode(event))
    {
    	kdDebug() << "canDecode(event) is true." << endl;
    }
    else
    {
	kdDebug() << "canDecode(event) is false." << endl;
    }
    //event->accept(QUriDrag::canDecode(event));
	event->accept(true);
}

void kdarDialog::dropEvent(QDropEvent *e)
{
	kdDebug() << "kdarDialog::dropEvent reached..." << endl;

	m_kdar->dropEvent(e);
}

/*void kdarDialog::customEvent( QCustomEvent * e )
{
	kdDebug() << "Received event." << endl;
        if ( e->type() == 1001 )  // It must be a kdarWarningEvent
        {
		kdDebug() << "It is a kdarWarningEvent." << endl;
		kdarWarningEvent* we = (kdarWarningEvent*)e;
		kdDebug() << "updating textbrowser..." << endl ;
		archiveTextBrowser->append(we->message());
		kdDebug() << "done." << endl;
        }
}
*/
bool kdarDialog::askUserQuestion ( QString userQuestion )
{
		int ret = KMessageBox::warningYesNo(0, userQuestion);
                bool answeredYes;
		answeredYes = (ret == KMessageBox::Yes);
		kdDebug() << "The user answered " << answeredYes << "." << endl;
		return answeredYes;
}

void kdarDialog::slotArchiveOpenFromDoubleClick(QListViewItem * item)
{
	kdDebug() << "kdarDialog::slotArchiveOpenFromDoubleClick reached." << endl;
	if (!item) return;
	if ( !item->parent() && !item->firstChild() )
	{
		//ensure that we have at least the top level of the listView populated
		m_kdar->slotArchiveList();
	}
	else
	{
		//If the item has children and it is closed, open it, and vice-versa
		item->setOpen(!item->isOpen());
	}
}

KDarConfig * kdarDialog::kdarConfig() const
{
	return m_kdc;
}

void kdarDialog::setKDarConfig( KDarConfig * kdarconfig )
{
	m_kdc = kdarconfig;
}

#include "kdardialog.moc"

