/*
 * Copyright (C) 2003,2004 Johnathan Burchill
 * <jkerrb@users.sourceforge.net>
 */

#include "kdar.h"
#include "openArchiveThread.h"

#include <kapplication.h>
#include <dcopclient.h>
#include <kaboutdata.h>
#include <kcmdlineargs.h>
#include <klocale.h>
#include <kconfig.h>

#include <qcstring.h>
#include <qfile.h>

#include <dar/libdar.hpp>
#include <config.h>

static const char *description =
    I18N_NOOP("KDar: the KDE Disk archiver.");
//
static const char *version = VERSION;

static KCmdLineOptions options[] =
{
	{ "create <profile>", I18N_NOOP( "Create an archive from a profile." ), 0 },
	{ "restore <profile>", I18N_NOOP( "Restore an archive from a profile." ), 0 },
    { "+[archive]", I18N_NOOP( "Archive to open." ), 0 },
	KCmdLineLastOption
};


int main(int argc, char **argv)
{

	//Are we running from the correct major version of libdar?
	libdar::U_I maj, min;
	libdar::get_version(maj, min);

    	if(maj != libdar::LIBDAR_COMPILE_TIME_MAJOR)
	{
		std::cout << "kdar: found libdar major version " << maj << "," << std::endl;
		std::cout << "         but kdar requires libdar major version " << libdar::LIBDAR_COMPILE_TIME_MAJOR << "." << std::endl;
		std::cout << "         Please obtain the correct version of libdar." << std::endl;
		return -1;
		// abort the program, we are not using the correct library
  	}

	//OK to procede! Let's get the compile-time capabilities:
	bool ea, largefile, nodump, special_alloc;
	libdar::U_I bits;

	libdar::get_compile_time_features(ea, largefile, nodump, special_alloc, bits/*, thread_safe*/);
/*	if (!thread_safe) 
	{
		std::cout << "kdar: KDar now requires a thread-safe dar library. Please upgrade your libdar." << std::endl;
		return -2;
	}
*/
	//convert the integer bit value to something the QT can handle.
	//The libdar::deci object is very useful indeed!
	libdar::deci deci_bits = libdar::deci(bits);
	QString text_bits = QString("\n   Maximum integer size: ") + QString(deci_bits.human().c_str())+QString(" bits");
	if (deci_bits.computer() == libdar::infinint(0)) text_bits = QString("\n   Maximum integer size: infinite");
	QString text_features = QString("Compiled-in capabilities:");
	QString text_ea = QString("\n   Extended attributes: ") + QString(ea ? "yes" : "no");
	QString text_large = QString("\n   Large files (> 2GB): ") + QString(largefile ? "yes" : "no");
	QString text_nodump = QString("\n   Ext3 \"nodump\" flag: ") + QString(nodump ? "yes" : "no");
	QString text_special_alloc = QString("\n   Special memory allocation: ") + QString(special_alloc ? "yes" : "no");
	//Don't need to display thread-safe status --- if the library is not thread-safe we will never get to this point anyway.
	//QString thread_safe_library = QString("\n   Thread-safe library: ") + QString(thread_safe ? "yes" : "no");
	//Libdar version:
	QString libdarVersion = QString("\n   Libdar version: ") + QString(libdar::deci(maj).human().c_str()+QString(".")+/*+libdar::deci(med).human().c_str()+QString(".")+*/libdar::deci(min).human().c_str());
	QString compile_features = text_features + text_ea + text_large + text_nodump + text_special_alloc + text_bits + libdarVersion;

	//Add the program, author, credit, etc., information to the "About" dialog:
	KAboutData about("kdar", I18N_NOOP("KDar"), version, description,
		KAboutData::License_GPL, "(C) 2003, 2004 Johnathan K. Burchill", compile_features, "http://kdar.sourceforge.net", "jkerrb@users.sourceforge.net");
	about.addAuthor( "Johnathan K. Burchill", "Development", "jkerrb@users.sourceforge.net", "http://members.shaw.ca/jkerrb");
	about.addCredit( "Denis Corbin", I18N_NOOP("For writing and maintaining libdar, \n and for answering my questions about the API."), 0, "http://sourceforge.net/projects/dar");
	about.addCredit( "Sven Bergner", I18N_NOOP("Code cleanups in kdar-1.0-beta1"), 0, 0);
	about.addCredit( "Simon Batchelor, Walter Gray, Andreas Pour, Christos Francheteau, Aad Rijnberg", I18N_NOOP("For reporting compilation problems/fixes."), 0, 0);
	about.addCredit( "WWagner", I18N_NOOP("For reporting directory include/exclude problem."), 0, 0);
	about.addCredit( "Alban Minassian", I18N_NOOP("For reporting directory filter bug, along with fix. Also for suggesting that \n a double-click on the archive name should list the contents in the file browser."), 0, 0);
	about.addCredit( "Eugenia Loli-Queru", I18N_NOOP("For reporting the crash due to missing directory in new archive paths.\nAlso for suggesting that a default backup directory be used,\nso users can enter archive names without a full path."), 0, 0);
	about.addCredit( "Daniel Nelson", I18N_NOOP( "For reporting the missing directory filters bug (#995601)" ), 0, 0 );
	about.addCredit( "Andrew E. Schulman", I18N_NOOP( "For reporting the unsaved execute command bug (#995780),\nthe endless-loop bug (#995772), \nand the missing archive name bug (#996539)" ), 0, 0 );
	//What's this do?
	KCmdLineArgs::init(argc, argv, &about);
	KCmdLineArgs::addCmdLineOptions(options);
	//Let's get the program going:
	KApplication app;

	// register it as a dcop client
	app.dcopClient()->registerAs(app.name(), false);

	//Session management stuff?
	if (app.isRestored())
	{
	RESTORE(kdar);
	}
	else
	{
		kdar *widget = new kdar;
		//Parse the commandline:
		KCmdLineArgs *args = KCmdLineArgs::parsedArgs();
		QCString createArchive = args->getOption( "create" );
		QCString restoreArchive = args->getOption( "restore" );
		
		if ( !createArchive.isEmpty() )
		{
			//Create an archive based on the profile
			std::cout << "main(): creating archive from " << createArchive << std::endl;
			return 0;
		}
		else if ( !restoreArchive.isEmpty() )
		{
			//Restore an archive based on the profile
			std::cout << "main(): restoring archive from " << restoreArchive << std::endl;
			return 0;
		}
		else if ( args->count() > 0 )
		{
			//Get the archive to open and open it:
			widget->slotArchiveChoose( QFile::decodeName( args->arg( 0 ) ) );
			widget->show();
			widget->slotArchiveList( false );
		}
		else
		{
			widget->show();
		}
	}
	//Run KDar!
	return app.exec();
}

