/***************************************************************************
 *   Copyright (C) 2004 by Johnathan Burchill                              *
 *   jkerrb@users.sourceforge.net                                          *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 ***************************************************************************/

#ifdef HAVE_CONFIG_H
#include <config.h>
#endif

#include "restoreArchiveThread.h"

#include "kdar.h"
#include "kdarConfig.h"
#include "kdardialog.h"
#include "kdarListViewItem.h"
#include "kdarSetCursorEvent.h"
#include "kdarStatusBarEvent.h"
#include "kdarWarningEvent.h"

#include <kcursor.h>
#include <kdebug.h>

#include <qstring.h>
#include <qlistview.h>

#include <dar/libdar.hpp>
#include <dar/erreurs.hpp>

#include <string>

extern bool libdarWorking;
extern libdar::archive *theArchive;
extern QString extractArchiveDirectory;

restoreArchiveThread::restoreArchiveThread( kdar * kdarWidget )
{
	widget = kdarWidget;
}

restoreArchiveThread::~restoreArchiveThread()
{
}

void restoreArchiveThread::run()
{
	libdarWorking = true;
	kdarSetCursorEvent *setWaitCursor = new kdarSetCursorEvent( KCursor::waitCursor());
    QApplication::postEvent( widget, setWaitCursor);
	kdarStatusBarEvent *statusEventBegin = new kdarStatusBarEvent( "Restoring files..." );	
	QApplication::postEvent( widget, statusEventBegin );
	
//	libdar::path fs_root();
	libdar::et_mask selection;// = new et_mask();
	selection.add_mask(libdar::bool_mask(true));
	libdar::et_mask subtree;// = new et_mask();
	libdar::ou_mask sub_include, sub_exclude;
	bool found_sub_include(false);

	//find which items are selected:
	QListViewItemIterator it( m_archiveFileView );
	while ( it.current() )
	{
		kdarListViewItem *item = static_cast<kdarListViewItem*>(it.current());
		++it;
		if ( item->isSelected() )
		{
			//Climb the listview tree to get the relative path of the item:
			kdarListViewItem *parent = item->parent();
			kdarListViewItem *tmpItem = item;
			QString addToExtractionList;
			while ( parent )
			{
				if ( tmpItem->parent() )
				{
					addToExtractionList.prepend( QString( "/" )+tmpItem->text( 0 ) );
					found_sub_include = true;
					tmpItem = tmpItem->parent();
				}
				parent = tmpItem->parent();
			}
			addToExtractionList = addToExtractionList.right(addToExtractionList.length()-1);
			sub_include.add_mask((libdar::simple_path_mask((m_kdc->extractArchiveDirectory()+addToExtractionList).latin1())));
			kdDebug() << "added " << (m_kdc->extractArchiveDirectory()+addToExtractionList).latin1() << " to extraction list." << endl;
		}
	}
	if (found_sub_include)
	{
		subtree.add_mask(sub_include);
		kdDebug() << "subtree got include extraction mask!" << endl;
	}
	else
	{
		subtree.add_mask(libdar::bool_mask(true));
		kdDebug() << "subtree is true (all files restored!)" << endl;
	}
	bool destroy(!m_kdc->noDelete());
	try
	{
		libdar::statistics st=op_extract(theArchive, m_kdc->extractArchiveDirectory().latin1(), selection, subtree, m_kdc->allowOverwrite(), m_kdc->warnOnOverwrite(), m_kdc->verbose(), destroy, m_kdc->restoreRecent(), m_kdc->systemEA(), m_kdc->userEA(), m_kdc->flatRestore(), m_kdc->ignoreID(), false, (libdar::infinint) m_kdc->hourShiftInt(), m_kdc->dryRun() );

		//Have to post this info as an event since this is not the GUI thread.
		QString extractInfo;
		extractInfo.append("<><><><><>\n\n");
		QString plural("s ");
		if (st.treated == libdar::infinint(1)) plural = " ";
		extractInfo.append(libdar::deci(st.treated).human().c_str()+QString(" file")+plural+QString("restored.\n"));
		plural = "s ";
		if (st.skipped == libdar::infinint(1)) plural = " ";
		extractInfo.append(libdar::deci(st.skipped).human().c_str()+QString(" file")+plural+QString("not restored (not saved in archive).\n"));
		plural = "s ";
		if (st.ignored == libdar::infinint(1)) plural = " ";
		extractInfo.append(libdar::deci(st.ignored).human().c_str()+QString(" file")+plural+QString("ignored (excluded by filters).\n"));
		plural = "s";
		if (st.tooold == libdar::infinint(1)) plural = "";
		extractInfo.append(libdar::deci(st.tooold).human().c_str()+QString(" file")+plural+QString(" less recent than the filesystem version"+plural+QString(".\n")));
		plural = "s ";
		if (st.errored == libdar::infinint(1)) plural = " ";
		extractInfo.append(libdar::deci(st.errored).human().c_str()+QString(" file")+plural+QString("failed to be restored (filesystem error).\n"));
		plural = "s were ";
		if (libdar::infinint(st.deleted) == libdar::infinint(1)) plural = " was ";
		extractInfo.append(libdar::deci(st.deleted).human().c_str()+QString(" file")+plural+QString("deleted from the filesystem.\n"));
		plural = "s were ";
		if (libdar::infinint(st.total()) == libdar::infinint(1)) plural = " was ";
		extractInfo.append(QString("A total of ")+libdar::deci(st.total()).human().c_str()+QString(" file")+plural+QString("considered.\n"));
		kdarWarningEvent *warningEvent = new kdarWarningEvent( extractInfo );	
		QApplication::postEvent( widget, warningEvent );
	}
	catch (libdar::Egeneric &e)
	{
		libdar::user_interaction_warning(e.get_message());
		libdarWorking = false;
	}
	kdarSetCursorEvent *setArrowCursor = new kdarSetCursorEvent( KCursor::arrowCursor());
    QApplication::postEvent( widget, setArrowCursor);
	kdarStatusBarEvent *statusEventEnd = new kdarStatusBarEvent( "Restoring files...done." );	
	QApplication::postEvent( widget, statusEventEnd );
	libdarWorking = false;

}

void restoreArchiveThread::setArchiveFileView( KListView * archiveFileView )
{
	m_archiveFileView = archiveFileView;
}

void restoreArchiveThread::setKDarConfig( KDarConfig * kdarconfig )
{
	m_kdc = kdarconfig;
}
